import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.media.j3d.*;

/**
 * Comportement personnalise du clavier :
 * Rotation du canon lorsqu'on presse les fleches droite-gauche du clavier
 */
public class CanonBehavior extends Behavior {

  // Groupe de transformation lie a l'action du clavier sur la direction
  // du canon
  private TransformGroup dirCanonTG = null;

  // Axe de tir des obus
  private Transform3D axeTir = new Transform3D();

  // Interpolateurs servant a decrire le mouvement des obus
  private PositionInterpolator posObus1Int = null;
  private PositionInterpolator posObus2Int = null;

  private Alpha posObus1Alpha = null;
  private Alpha posObus2Alpha = null;

  // Transformation associee au TransformGroup dirCanonTG
  private Transform3D rotation = new Transform3D();

  // Condition qui va declencher le stimulus
  private WakeupCondition wakeupCondition =
      new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);

  // Angle de la rotation du canon (degres)
  private float angle;

  /**
   * Constructeur de notre comportement lie au canon
   * @param dirCanonTG TransformGroup permettant de modifier la direction
   * du canon
   */
  public CanonBehavior(TransformGroup dirCanonTG,
                       Transform3D axeTir,
                       PositionInterpolator posObus1Int,
                       PositionInterpolator posObus2Int,
                       Alpha posObus1Alpha,
                       Alpha posObus2Alpha) {

    this.dirCanonTG = dirCanonTG;
    this.axeTir = axeTir;
    this.posObus1Int = posObus1Int;
    this.posObus2Int = posObus2Int;
    this.posObus1Alpha = posObus1Alpha;
    this.posObus2Alpha = posObus2Alpha;

    angle = 0;
  }

  /**
   * Initialisation du comportement :
   * La methode processStimulus() sera appelee a chaque fois qu'une touche
   * du clavier sera pressee
   */
  public void initialize() {
    this.wakeupOn(wakeupCondition);
  }

  /**
   * Methode appelee a chaque fois que l'on appuie sur une touche du clavier
   * @param criteria critere ayant declenche le stimulus (ici un evenement
   * AWT correspondant a une pression sur une touche du clavier)
   */
  public void processStimulus(Enumeration criteria) {
    WakeupCriterion critere;
    AWTEvent[] events;
    KeyEvent evt;

    // On boucle sur les criteres ayant declenche le comportement
    while (criteria.hasMoreElements()) {
      // On recupere le premier critere de l'enumeration
      critere = (WakeupCriterion)criteria.nextElement();

      // On ne traite que les criteres correspondant a un evenement AWT
      if (critere instanceof WakeupOnAWTEvent) {
        // On recupere le tableau des evements AWT correspondant au critere
        events = ((WakeupOnAWTEvent)critere).getAWTEvent();

        if (events.length > 0) {
          // On recupere l'evement (le tableau d'evenements ne contient en
          // fait qu'un seul evenement)
          evt = (KeyEvent)events[events.length-1];

          // Traitement au cas par cas selon la touche du clavier pressee
          switch(evt.getKeyCode()) {
            // Fleche droite : rotation du canon vers la droite
            case KeyEvent.VK_RIGHT:
              angle = angle - 3;
              if (angle < -45)
                angle = -45;

              rotation.rotZ(angle * Math.PI / 180f);
              dirCanonTG.setTransform(rotation);
              break;

            // Fleche gauche : rotation du canon vers la gauche
            case KeyEvent.VK_LEFT:
              angle = angle + 3;
              if (angle > 45)
                angle = 45;

              rotation.rotZ(angle * Math.PI / 180f);
              dirCanonTG.setTransform(rotation);
              break;

            // Barre d'espace : on lance un obus ou deux obus
            case KeyEvent.VK_SPACE:
              double angleRad = ((angle+90.0) * Math.PI) / 180.0;

              axeTir.rotZ(angleRad);

              if (posObus1Alpha.isPaused()) {
                posObus1Int.setTransformAxis(axeTir);
                posObus1Alpha.resume();
              }

              if (posObus2Alpha.isPaused() && (posObus1Alpha.value() > 0.5)) {
                posObus2Int.setTransformAxis(axeTir);
                posObus2Alpha.resume();
              }

              if (posObus1Alpha.finished() && (posObus2Alpha.value() > 0.5)) {
                posObus1Int.setTransformAxis(axeTir);
                posObus1Alpha.setStartTime(System.currentTimeMillis());
              }

              if (posObus2Alpha.finished() && (posObus1Alpha.value() > 0.5)) {
                posObus2Int.setTransformAxis(axeTir);
                posObus2Alpha.setStartTime(System.currentTimeMillis());
              }
              break;
          }  // fin switch(evt.getKeyCode())
        }  // fin if (events.length > 0)
      }  // fin if (wakeup instanceof WakeupOnAWTEvent)
    }  // fin while (criteria.hasMoreElements())

    // Une fois le stimulus traite, on reinitialise le comportement
    this.wakeupOn(wakeupCondition);
  }
}
