import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.media.j3d.*;

/**
 * Comportement personnalise du clavier :
 * Rotation du canon lorsqu'on presse les fleches droite-gauche du clavier
 */
public class CanonBehavior extends Behavior {

  // Groupe de transformation lie a l'action du clavier sur la direction
  // du canon
  private TransformGroup dirCanonTG = null;

  // Axe de tir des obus
  private Transform3D axeTir = new Transform3D();

  // Interpolateurs servant a decrire le mouvement des obus
  private PositionInterpolator posObus1Int = null;
  private PositionInterpolator posObus2Int = null;

  private Alpha posObus1Alpha = null;
  private Alpha posObus2Alpha = null;

  // Transformation associee au TransformGroup dirCanonTG
  private Transform3D rotation = new Transform3D();

  // Sert a declencher le stimulus si un des criteres
  // (touche pressee OU relachee) est verifie
  private WakeupOr oredCriteria = null;

  // Angle de la rotation du canon (degres)
  private float angle;

  private KeyActionThread keyActionThread = new KeyActionThread();
  private boolean keyPressedLeftArr, keyPressedRightArr, keyPressedSpace;

  /**
   * Constructeur de notre comportement lie au canon
   * @param dirCanonTG TransformGroup permettant de modifier la direction
   * du canon
   */
  public CanonBehavior(TransformGroup dirCanonTG,
                       Transform3D axeTir,
                       PositionInterpolator posObus1Int,
                       PositionInterpolator posObus2Int,
                       Alpha posObus1Alpha,
                       Alpha posObus2Alpha) {

    this.dirCanonTG = dirCanonTG;
    this.axeTir = axeTir;
    this.posObus1Int = posObus1Int;
    this.posObus2Int = posObus2Int;
    this.posObus1Alpha = posObus1Alpha;
    this.posObus2Alpha = posObus2Alpha;

    keyPressedLeftArr = false;
    keyPressedRightArr = false;
    keyPressedSpace = false;
    angle = 0;
  }

  /**
   * Initialisation du comportement :
   * La methode processStimulus() sera appelee a chaque fois qu'une touche
   * du clavier sera pressee OU relachee
   */
  public void initialize() {
    // Liste des criteres qui declenchent le stimulus pour l'action sur le
    // canon
    WakeupCriterion[] criteres = new WakeupCriterion[2];

    // Premier critere : on presse une touche du clavier
    WakeupOnAWTEvent keyPressedCritere =
        new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);

    // Deuxieme critere : on relache une touche du clavier
    WakeupOnAWTEvent keyReleasedCritere =
        new WakeupOnAWTEvent(KeyEvent.KEY_RELEASED);

    criteres[0] = keyPressedCritere;
    criteres[1] = keyReleasedCritere;

    // Stimulus declenche si l'un des 2 criteres ci-dessus est verifie
    oredCriteria = new WakeupOr(criteres);
    this.wakeupOn(oredCriteria);
  }

  /**
   * Methode appelee a chaque fois que l'on appuie sur une touche du clavier
   * @param criteria critere ayant declenche le stimulus (ici un evenement
   * AWT correspondant a une pression sur une touche du clavier)
   */
  public void processStimulus(Enumeration criteria) {
    WakeupCriterion critere;
    AWTEvent[] events;

    // On boucle sur les criteres ayant declenche le comportement
    while (criteria.hasMoreElements()) {
      // On recupere le premier critere de l'enumeration
      critere = (WakeupCriterion)criteria.nextElement();

      // On ne traite que les criteres correspondant a un evenement AWT
      if (critere instanceof WakeupOnAWTEvent) {
        // On recupere le tableau des evements AWT correspondant au critere
        events = ((WakeupOnAWTEvent)critere).getAWTEvent();

        if (events.length > 0) {
          // On recupere l'evement (le tableau d'evenements ne contient en
          // fait qu'un seul evenement)
          KeyEvent keyEvent = (KeyEvent)events[events.length-1];

          if (keyEvent.getID() == KeyEvent.KEY_PRESSED) {

            // Traitement au cas par cas selon la touche du clavier pressee
            switch(keyEvent.getKeyCode()) {
              // Fleche droite : rotation du canon vers la droite
              case KeyEvent.VK_LEFT:
                keyPressedLeftArr = true;
                break;

              // Fleche gauche : rotation du canon vers la gauche
              case KeyEvent.VK_RIGHT:
                keyPressedRightArr = true;
                break;

              // Barre d'espace : on lance un obus ou deux obus
              case KeyEvent.VK_SPACE:
                keyPressedSpace = true;
                break;
            }  // fin switch(evt.getKeyCode())
          }  // fin if (keyEvent.getID() == KeyEvent.KEY_PRESSED)

          else if (keyEvent.getID() == KeyEvent.KEY_RELEASED) {

            // Traitement au cas par cas selon la touche du clavier pressee
            switch(keyEvent.getKeyCode()) {
              // Fleche droite : rotation du canon vers la droite
              case KeyEvent.VK_LEFT:
                keyPressedLeftArr = false;
                break;

              // Fleche gauche : rotation du canon vers la gauche
              case KeyEvent.VK_RIGHT:
                keyPressedRightArr = false;
                break;
            }  // fin switch(evt.getKeyCode())
          }  // fin else if (keyEvent.getID() == KeyEvent.KEY_RELEASED)

          if (!keyActionThread.isAlive()) {
            keyActionThread = new KeyActionThread();
            keyActionThread.start();
          }

        }  // fin if (events.length > 0)
      }  // fin if (wakeup instanceof WakeupOnAWTEvent)
    }  // fin while (criteria.hasMoreElements())

    // Une fois le stimulus traite, on reinitialise le comportement
    this.wakeupOn(oredCriteria);
  }

  class KeyActionThread extends Thread {

    public void run() {

      while(true) {

        try {
          Thread.sleep(20);
        }
        catch (InterruptedException ex) {
        }

        // Traitement au cas par cas selon la touche du clavier pressee
        // Fleche gauche pressee : rotation du canon vers la gauche
        if (keyPressedLeftArr) {
          angle = angle + 3;
          if (angle > 45)
            angle = 45;

          rotation.rotZ(angle * Math.PI / 180f);
          dirCanonTG.setTransform(rotation);
        }  // fin if (keyPressedLeftArr)

        // Fleche droite pressee : rotation du canon vers la droite
        if (keyPressedRightArr) {
          angle = angle - 3;
          if (angle < -45)
            angle = -45;

          rotation.rotZ(angle * Math.PI / 180f);
          dirCanonTG.setTransform(rotation);
        }  // fin if (keyPressedRightArr)

        // Barre d'espace pressee : on lance un obus ou deux obus
        if (keyPressedSpace) {
          double angleRad = ((angle+90.0) * Math.PI) / 180.0;

          axeTir.rotZ(angleRad);

          if (posObus1Alpha.isPaused()) {
            // Pour placer l'obus au bout du canon juste avant le tir
            posObus1Int.setStartPosition(1);
            posObus1Int.setTransformAxis(axeTir);
            posObus1Alpha.resume();
          }

          if (posObus2Alpha.isPaused() && (posObus1Alpha.value() > 0.5)) {
            // Pour placer l'obus au bout du canon juste avant le tir
            posObus2Int.setStartPosition(1);
            posObus2Int.setTransformAxis(axeTir);
            posObus2Alpha.resume();
          }

          if (posObus1Alpha.finished() && (posObus2Alpha.value() > 0.5)) {
            posObus1Int.setTransformAxis(axeTir);
            posObus1Alpha.setStartTime(System.currentTimeMillis());
          }

          if (posObus2Alpha.finished() && (posObus1Alpha.value() > 0.5)) {
            posObus2Int.setTransformAxis(axeTir);
            posObus2Alpha.setStartTime(System.currentTimeMillis());
          }

          keyPressedSpace = false;
        }  // fin if (keyPressedSpace)

        if (!keyPressedLeftArr && !keyPressedRightArr) {
          break;
        }
      }  // fin while(true)
    }  // fin run()
  }
}
